/*
 * NXPower.h - NXPower class provides information about UPower service
 *             objects: laptop LID, battery. Should be used to initiate
 *             changing of power profile of system(tuned) and actions:
 *             shutdown, restart, hibernate and sleep.
 *
 * Copyright 2016, Serg Stoyan
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifdef WITH_UPOWER

#import <Foundation/NSObject.h>

#include <upower.h>

@class NSString;

@interface NXPower : NSObject
{
  UpClient *upower_client;
}
// Battery information
+ (unsigned int)batteryLife;
+ (unsigned char)batteryPercent;
+ (BOOL)isUsingBattery;

// LID information
+ (BOOL)hasLid;
+ (BOOL)isLidClosed;
- (BOOL)hasLid;
- (BOOL)isLidClosed;
  
// Monitor for D-Bus messages
- (void)startEventsMonitor;
- (void)stopEventsMonitor;

@end

extern NSString *NXPowerLidDidChangeNotification;

#endif //WITH_UPOWER
